#!/usr/bin/ruby
# $Id: GalateaBase.rb,v 1.3 2008/12/19 04:53:47 nishimoto Exp $
#
# Galatea Submodule base class
# by Takuya NISHIMOTO (nishi@hil.t.u-tokyo.ac.jp)
#

require 'thread'
require 'timeout'

# grobal method

def print_out(str)
  puts str
  STDOUT.flush
end

# class def.

class GalateaBase
  def initialize
    @modulename = "GalateaBase"
    @debug = false
    @myslot = {}
    @mymacro = {}
    @myprop = {}
    @mycount = {}
    #
    @myslot["Run"] = "DEAD" # or "LIVE"
    @myslot["ProtocolVersion"] = "ProtocolVersion 1.0"
    @myslot["ModuleVersion"] = "ModuleVersion 1.0"
    @alive_count = 1
    @check_alive = false
  end

  attr_accessor :modulename, :debug
  attr_accessor :check_alive

  def send_set(mod, slot, str)
    puts "to \@#{mod} set #{slot} = #{str}"
    STDOUT.flush
  end

  def print_out(str)
    puts str
    STDOUT.flush
  end

  def print_debug(str)
    if @debug
      #STDERR.puts @modulename + ": " + str
      #STDERR.flush
      print_out "to \@MON set SysLogText = #{modulename} #{str}"
    end
  end

  def print_tell(str)
    puts "tell " + str
    STDOUT.flush
  end

  def print_rep(str)
    puts "rep " + str
    STDOUT.flush
  end

  def do_gets(rs = $/)
    if @check_alive
      s = nil
      while s == nil
        begin
          timeout(5.0) do
            s = $stdin.gets(rs)
          end
        rescue TimeoutError
          print_debug("alive_count #{@alive_count}")
          @alive_count += 1
        end
      end
      return s
    else
      $stdin.gets(rs)
    end
  end

  def run
    while true
      while cmd = do_gets
        case cmd
        when /^\s*set\s+([^\s=]*)\s*=\s*(.*)$/
          # set XXX = xxxx
          @myslot[$1] = $2
          do_set $1, $2
        when /^\s*prop\s+([^\s=]*)\s*=\s*(.*)$/
          # prop XXX.attr = xxx
          @myprop[$1] = $2
          do_prop $1, $2
        when /^\s*inq\s+(.*)$/
          # inq
          print_rep "#{$1} = #{@myslot[$1]}"
        when /^\s*From\s+\@(\S+)\s(.*)$/
          # (broadcast only) From @(XXX) (xxxxxxxx)
          accept_from $1, $2
        when /^\s*to\s+\@(\S+)\s(.*)$/
          # (broadcast only) to @(XXX) (xxxxxxxx)
          accept_to $1, $2
        end
        # not implemented:
        #
        # set XXX < filename
        #
        # set XXX << END
        # xxxx
        # END
      end
    end
  end

  def do_set(slot, arg)
    if slot == "Run" 
      if arg == "INIT"
        @myslot["Run"] = "LIVE"
      end
      if arg == "EXIT"
        @myslot["Run"] = "DEAD"
      end
    end
  end

  def do_prop(prop, arg)
  end

  def accept_from(mod, str)
  end

  def accept_to(mod, str)
  end

end

#############################################################

class GalateaBaseThread < GalateaBase
  def initialize
    super
    @modulename = "GalateaBaseThread"
    @mutex = Mutex.new
    @input_queue = Array.new
  end

  attr_accessor :modulename, :debug

  def run
    Thread.start do
      while true
        sleep 0.0001
        while @input_queue.length > 0 do
          cmd = nil
          @mutex.synchronize do
            cmd = @input_queue.delete_at(0)
          end

          # set XXX = xxxx
          if /^\s*set\s+([^\s=]*)\s*=\s*(.*)$/ =~ cmd
            @myslot[$1] = $2
            do_set $1, $2
          end

          # set XXX < filename
          # not implemented

          # prop XXX.attr = xxx
          if /^\s*prop\s+([^\s=]*)\s*=\s*(.*)$/ =~ cmd
            @myprop[$1] = $2
            do_prop $1, $2
          end

          # set XXX << END
          # xxxx
          # END
          # not implemented

          # inq
          if /^\s*inq\s+(.*)$/ =~ cmd
            print_rep "#{$1} = #{@myslot[$1]}"
          end

          # (broadcast only) From @(XXX) (xxxxxxxx)
          if /^\s*From\s+\@(\S+)\s(.*)$/ =~ cmd
            accept_from $1, $2
          end

          # (broadcast only) to @(XXX) (xxxxxxxx)
          if /^\s*to\s+\@(\S+)\s(.*)$/ =~ cmd
            accept_to $1, $2
          end
        end
      end
    end

    while true
      while gets do
        @mutex.synchronize do
          @input_queue.push($_)
        end
        sleep 0.0001
      end
    end

  end

end


# end of file
